@@ -2,7 +2,8 @@
 /**
  * PokoX - Super Light Weight MVC Controller for PHP.
  *
- * @version 1.0
+ * @package PokoX
+ * @version 1.3.0
  * @copyright Copyright (c) 2003 - 2005 Akira Kaze
  *
  * Character set is EUC-JP.
@@ -11,10 +12,16 @@
  * distributed with this source code. If the COPYRIGHT.txt file is missing, 
  * please visit the PokoX homepage: http://www.glamenv-septzen.net/
  *
- * $Id: pokox.inc.php,v 1.3 2005/07/10 20:58:28 fenjin Exp $
+ * $Id: pokox.inc.php,v 1.10 2005/09/10 09:59:44 fenjin Exp $
  */
 
 if(defined('_POKOX_KERNEL')) return;
+/**
+ * pokox.inc.php ɤ߹ɻߥե饰
+ *
+ * @since 1.0
+ * @var bool
+ */
 define('_POKOX_KERNEL', 1);
 
 /*
@@ -25,12 +32,22 @@ define('_POKOX_KERNEL', 1);
  * ϻԲ!
  */
 
+if(!defined('POKOX_ACI_KEY')) {
+/**
+ * ACI$_POKOXФΥʸ
+ *
+ * @since 1.2
+ * @var string
+ */
+define('POKOX_ACI_KEY', 'pokox_aci');
+}
+
 if(!defined('POKOX_STORY_KEY')) {
 /**
  * ȡ꡼ФΥʸ
  *
  * @since 1.0
- * @type string
+ * @var string
  */
 define('POKOX_STORY_KEY', 'pokox_story');
 }
@@ -40,7 +57,7 @@ if(!defined('POKOX_STORY_KEY_PREV')) {
  * ΥڡΥȡ꡼ФΥʸʤС
  *
  * @since 1.0
- * @type string
+ * @var string
  */
 define('POKOX_STORY_KEY_PREV', 'pokox_story_prev');
 }
@@ -50,7 +67,7 @@ if(!defined('POKOX_DIR')) {
  * PokoX饤֥ǥ쥯ȥ
  *
  * @since 1.0
- * @type string
+ * @var string
  */
 define('POKOX_DIR', dirname(__FILE__));
 }
@@ -61,7 +78,7 @@ if(!defined('POKOX_STORY_DIR')) {
  * PokoXȡ꡼饹ǥ쥯ȥʥǥեȡ
  *
  * @since 1.0
- * @type string
+ * @var string
  */
 define('POKOX_STORY_DIR', realpath(POKOX_DIR."/story/"));
 }
@@ -71,7 +88,7 @@ if(!defined('POKOX_STORY_AUTOLOAD')) {
  * ȡ꡼饹μưɤ߹ߵǽON/OFF
  *
  * @since 1.0
- * @type string
+ * @var string
  */
 define('POKOX_STORY_AUTOLOAD', 1);
 }
@@ -81,7 +98,7 @@ if(!defined('POKOX_ERROR_LEVEL')) {
  * PokoXνϤ륨顼٥
  *
  * @since 1.0
- * @type string
+ * @var string
  */
 define('POKOX_ERROR_LEVEL', E_USER_WARNING);
 }
@@ -102,7 +119,7 @@ class PokoX
 	 *
 	 * @access public
 	 * @since 1.0
-	 * @type PokoXRenderer
+	 * @var PokoXRenderer
 	 */
 	var $renderer = null;
 
@@ -111,7 +128,7 @@ class PokoX
 	 *
 	 * @access private
 	 * @since 1.0
-	 * @type array
+	 * @var array
 	 */
 	var $_modules;
 
@@ -121,7 +138,7 @@ class PokoX
 	 *
 	 * @access private
 	 * @since 1.0
-	 * @type array
+	 * @var array
 	 */
 	var $_module_stacks;
 
@@ -130,7 +147,7 @@ class PokoX
 	 *
 	 * @access private
 	 * @since 1.0
-	 * @type bool
+	 * @var bool
 	 */
 	var $_dispatched = false;
 
@@ -139,7 +156,7 @@ class PokoX
 	 *
 	 * @access private
 	 * @since 1.0
-	 * @type bool
+	 * @var bool
 	 */
 	var $_escaped = false;
 
@@ -148,10 +165,28 @@ class PokoX
 	 *
 	 * @access private
 	 * @since 1.0
-	 * @type bool
+	 * @var bool
 	 */
 	var $_ignore_renderer = false;
 
+	/**
+	 * Template꥽̾(PokoXModule)
+	 *
+	 * @access private
+	 * @since 1.2
+	 * @var string
+	 */
+	var $_template_resource = null;
+
+	/**
+	 * HTML¸Хåե
+	 *
+	 * @access private
+	 * @since 1.3
+	 * @var string
+	 */
+	var $_contents = null;
+
 	/**
 	 * PokoX󥹥󥹤Singletonǥѥ롣
 	 *
@@ -185,12 +220,12 @@ class PokoX
 	 * PokoX ؿ
 	 *
 	 * @param string ȡ꡼ͥ졼饹̾
-	 * @param string 饯饹̾
+	 * @param string 饯饹̾(Ѥʤϻ̵)
 	 * @return string 
 	 * @access public
 	 * @since 1.0
 	 */
-	function _run($StoryClassName, $RendererClassName)
+	function _run($StoryClassName, $RendererClassName = null)
 	{
 
 		// Create Singleton Instance
@@ -218,17 +253,23 @@ class PokoX
 		}
 
 		// Load Renderer Class
-		if(!class_exists($RendererClassName)) {
+		if(is_string($RendererClassName) &&
+			!class_exists($RendererClassName)) {
 			user_error(
 				"Not found Renderer Class '$RendererClassName'",
 				POKOX_ERROR_LEVEL
 				);
 		}
+		// if Not Speciied Renderer Class, use VarDumpRenderer as Dummy.
+		// Renderer Class Instance is created, but parseTemplate() 
+		// method doesn't execute at end.
+		$_RendererClassName = (is_null($RendererClassName)) ? 
+			'PokoXVarDumpRenderer' : $RendererClassName;
 
 		// Create Story Instance
 		$story =& new $StoryClassName();
 		// Create Renderer Instance
-		$pokox->renderer =& new $RendererClassName();
+		$pokox->renderer =& new $_RendererClassName();
 		// Start Story
 		$story->start($pokox);
 		// (Re-)Declare $_POKOX Variable as Global Variable
@@ -245,52 +286,91 @@ class PokoX
 			call_user_func("POKOX_STORY_PREHOOK", $pokox);
 		}
 
-		// Get Current Story
+		// Get Current Story and ACI
 		$pokox_story = (isset($_POKOX[POKOX_STORY_KEY])) ? 
 			$_POKOX[POKOX_STORY_KEY] : "*";
-		// Get Current ACI
-		$pokox_aci = (isset($_POKOX["pokox_aci"])) ? 
-			$_POKOX["pokox_aci"] : "*";
+		$pokox_aci = (isset($_POKOX[POKOX_ACI_KEY])) ? 
+			$_POKOX[POKOX_ACI_KEY] : "*";
 
-		// Construct Module Chain
-		$pokox->_modules = $pokox->_parsePokoXDirectives(
-			$pokox_aci, $pokox_story, "POKOX_MODULE_STORY");
 		// Invoke Module Chain
-		$pokox->_invokeModuleChain();
+		$pokox->_invokeModuleChain($pokox_story, $pokox_aci);
 
 		// Hook AFTER Execute Chain
 		if(function_exists("POKOX_STORY_POSTHOOK")) {
 			call_user_func("POKOX_STORY_POSTHOOK", $pokox);
 		}
 
+		// Update Current Story (for security reason, don't update ACI)
+		$pokox_story = (isset($_POKOX[POKOX_STORY_KEY])) ? 
+			$_POKOX[POKOX_STORY_KEY] : "*";
+
 		// Assign Auto Template Variables
+		$pokox->renderer->assign(
+			'POKOX_ACI_KEY', POKOX_ACI_KEY);
 		$pokox->renderer->assign(
 			'POKOX_STORY_KEY', POKOX_STORY_KEY);
 		$pokox->renderer->assign(
 			'POKOX_STORY_KEY_PREV', POKOX_STORY_KEY_PREV);
 		$pokox->renderer->assign(
-			'POKOX_ACI', @$_POKOX['pokox_aci']);
+			'POKOX_ACI', @$_POKOX[POKOX_ACI_KEY]);
 		$pokox->renderer->assign(
 			'POKOX_STORY', $_POKOX[POKOX_STORY_KEY]);
 		$pokox->renderer->assign(
 			'POKOX_STORY_PREV', @$_POKOX[POKOX_STORY_KEY_PREV]);
 		
-		// escapeƤʤƤƤ椬ƤС
+		/*
+		 * Not Escaped, or, required renderer control, Execute PokoX Renderer.
+		 */
 		if((!$pokox->_escaped) || 
 			($pokox->_escaped && !$pokox->_ignore_renderer)) {
+
+			// if $this->_template_resource is not null, it overwrites 
+			// template story result.
+			if(is_null($pokox->_template_resource)) {
+				$_trname = $pokox->_parsePokoXDirectives(
+					$pokox_aci, $pokox_story, "POKOX_TEMPLATE_STORY");
+			} else {
+				$_trname = $pokox->_template_resource;
+			}
+
 			// Execute PokoXRenderer & Get Output
-			$contents = $pokox->renderer->parseTemplate(
-				$pokox->_parsePokoXDirectives(
-					$pokox_aci, 
-					$pokox_story,
-					"POKOX_TEMPLATE_STORY")
-				);
+			if(is_null($RendererClassName) || is_null($_trname)) {
+				$contents = null;
+			} else {
+				$contents = $pokox->renderer->parseTemplate($_trname);
+				$pokox->setContents($contents);
+			}
 		}
 
 		// End Story
 		$story->end($pokox);
 
-		return $contents;
+//		return $contents;
+		return $pokox->getContents();
+	}
+
+	/**
+	 * HTMLϤ򥯥饹Хåե¸롣
+	 *
+	 * @param string HTML
+	 * @access public
+	 * @since 1.3
+	 */
+	function setContents(&$contents)
+	{
+		$this->_contents = $contents;
+	}
+
+	/**
+	 * 饹Хåե¸ƤHTMLϤ롣
+	 *
+	 * @return string HTML
+	 * @access public
+	 * @since 1.3
+	 */
+	function &getContents()
+	{
+		return $this->_contents;
 	}
 
 	/**
@@ -310,6 +390,9 @@ class PokoX
 	 */
 	function _parsePokoXDirectives($aci, $story, $directives)
 	{
+		if(!isset($GLOBALS[$directives]) || is_null($GLOBALS[$directives])) {
+			return null;
+		}
 		foreach($GLOBALS[$directives] as $directive => $value) {
 			switch($directive) {
 				case "$aci.$story" :
@@ -321,19 +404,115 @@ class PokoX
 		}
 	}
 
+	/**
+	 * PokoXModule饹եΥѥ롣
+	 *
+	 * @param string ǥ쥯ȥ
+	 * @param string 饹̾
+	 * @return ǥ쥯ȥ.'/'.饹̾.'.class.php'
+	 * @access private
+	 * @since 1.2
+	 */
+	function __formatModuleClassFile($path, $class_name)
+	{
+		return realpath(sprintf('%s/%s.class.php', $path, $class_name));
+	}
+
+	/**
+	 * PokoXModule饹ɤ롣$POKOX_MODULE_AUTOLOAD_PATH˥⥸塼
+	 * 饹եΤǥ쥯ȥ(ʣξ)ꤵƤ
+	 * ˡưŪ˥饹եõrequire롣
+	 *
+	 * @param string 饹̾
+	 * @return bool 饹ɺѤߤ饹եrequiretrue.
+	 *              饹ե뤬Ĥʤfalse.
+	 * @access private
+	 * @since 1.2
+	 */
+	function _loadModuleClass($class_name)
+	{
+		if(class_exists($class_name)) return true;
+		if(!isset($GLOBALS['POKOX_MODULE_AUTOLOAD_PATH'])) return true;
+
+		$paths = $GLOBALS['POKOX_MODULE_AUTOLOAD_PATH'];
+
+		$curdir_fn = PokoX::__formatModuleClassFile('.', $class_name);
+		if(file_exists($curdir_fn)) {
+			require_once($curdir_fn);
+			return true;
+		}
+		if(is_string($paths)) {
+			// single directory path
+			$sdp_fn = PokoX::__formatModuleClassFile($paths, $class_name);
+			if(file_exists($sdp_fn)) {
+				require_once($sdp_fn);
+				return true;
+			}
+		} else if(is_array($paths)) {
+			// multiple directory path
+			foreach($paths as $path) {
+				$mdp_fn = PokoX::__formatModuleClassFile($path, $class_name);
+				if(file_exists($mdp_fn)) {
+					require_once($mdp_fn);
+					return true;
+				}
+			}
+		}
+		return false;
+	}
+
+	/**
+	 * ⥸塼᥽å
+	 *
+	 * @param string PokoXθߤΥȡ꡼
+	 * @param string PokoXθߤACI
+	 * @access private
+	 * @since 1.2
+	 */
+	function _initModuleChain($pokox_story, $pokox_aci)
+	{
+		unset($this->_modules);
+		$this->_modules = array();
+		if(isset($GLOBALS['POKOX_MODULE_STORY_EX'])) {
+			// Extended(Filter Featured) Module Chain Construction
+			reset($GLOBALS['POKOX_MODULE_STORY_EX']);
+			foreach($GLOBALS['POKOX_MODULE_STORY_EX'] as $key => $value) {
+				// Separate aci and story directive
+				$_pos = strpos($key, '.');
+				$_aci = substr($key, 0, $_pos);
+				$_story = substr($key, $_pos+1);
+				if(($_aci == $pokox_aci) || ($_aci == "*")) {
+					if(preg_match($_story, $pokox_story)) {
+						$this->_modules = array_merge(
+							$this->_modules, $value);
+					}
+				}
+			}
+		} else {
+			// Normal Module Chain Construction
+			$this->_modules = $this->_parsePokoXDirectives(
+				$pokox_aci, $pokox_story, "POKOX_MODULE_STORY");
+			if(!isset($this->_modules)) {
+				// If not specified current story, initialize as empty array.
+				$this->_modules = array();
+			}
+		}
+		reset($this->_modules);
+	}
+
 	/**
 	 * ⥸塼¹ԥ᥽å
 	 *
+	 * @param string PokoXθߤΥȡ꡼
+	 * @param string PokoXθߤACI
 	 * @access private
 	 * @since 1.0
 	 */
-	function _invokeModuleChain()
+	function _invokeModuleChain($pokox_story, $pokox_aci)
 	{
-		if(!isset($this->_modules)) {
-			// ⤷⳺ȡ꡼¸ߤʤСǽƤ
-			$this->_modules = array();
-		}
-		reset($this->_modules);
+		global $_POKOX;
+		// Initialize Module Chain
+		$this->_initModuleChain($pokox_story, $pokox_aci);
 		do {
 			// dispatchե饰̵Ƥ
 			if($this->_dispatched) {
@@ -347,33 +526,25 @@ class PokoX
 				} else {
 					$mid = null;
 				}
+				$this->_loadModuleClass($class_name);
 				$module =& new $class_name();
-				$module->execute($this, $params, $mid);
+				$_result = $module->execute($this, $params, $mid);
+				if(is_string($_result)) {
+					$this->_template_resource = $_result;
+				}
 				if($mid) {
 					// ⥸塼IDؼƤСåѤǤ
 					$this->_module_stacks[$mid] =& $module;
 				}
-				// escapedե饰ͭˤʤäƤХ롼פȴ롣
-				if($this->_escaped) break;
+				// escapedե饰ͭˤʤäƤнλ
+				if($this->_escaped) return;
 				// dispatchե饰ͭˤʤäƤХ롼פȴ롣
 				if($this->_dispatched) break;
 			}
-			if($this->_escaped) {
-				break;
-			}
 			if($this->_dispatched) {
-				// ⥸塼κƹ
-				global $_POKOX;
-				unset($this->_modules);
-				$this->_modules = array();
-				$this->_modules = $this->_parsePokoXDirectives(
-					$_POKOX['pokox_aci'], 
-					$_POKOX[POKOX_STORY_KEY],
-					"POKOX_MODULE_STORY");
-				if(!isset($this->_modules)) {
-					$this->_modules = array();
-				}
-				reset($this->_modules);
+				// Initialize Module Chain
+				$this->_initModuleChain($_POKOX[POKOX_STORY_KEY],
+					$_POKOX[POKOX_ACI_KEY]);
 			}
 		// dispatchͭƤС٥⥸塼¹Ԥ롣
 		} while($this->_dispatched);
@@ -416,19 +587,46 @@ class PokoX
 	 */
 	function &getStackedModule($mid)
 	{
-		if(isset($this->_module_stacks[$mid])) {
-			return $this->_module_stacks[$mid];
+		$pokox =& PokoX::getInstance();
+		if(isset($pokox->_module_stacks[$mid])) {
+			return $pokox->_module_stacks[$mid];
 		} else {
 			return null;
 		}
 	}
+
+	/**
+	 * ߤΥ⥸塼륹åλȤ֤
+	 *
+	 * @return array ߤΥ⥸塼륹åλ
+	 * @access public
+	 * @since 1.2
+	 */
+	function &getStackedModules()
+	{
+		$pokox =& PokoX::getInstance();
+		return $pokox->_module_stacks;
+	}
+
+	/**
+	 * ߤΥ⥸塼򥳥ԡ֤
+	 *
+	 * @return array ߤΥ⥸塼Υԡ
+	 * @access public
+	 * @since 1.2
+	 */
+	function getModuleChain()
+	{
+		$pokox =& PokoX::getInstance();
+		return $pokox->_modules;
+	}
 }
 
 /**
  * PokoXݡ˥饹
  *
  * @author Akira Kaze <feng-jing-gsyc-2s@glamenv-septzen.net>
- * @package pokox
+ * @package PokoX
  * @since 1.0
  */
 /* interface or abstract */
@@ -438,7 +636,7 @@ class PokoXRenderer
 	 * ºݤΥƥץ졼ȥ֥
 	 *
 	 * @access public
-	 * @type object
+	 * @var object
 	 * @since 1.0
 	 */
 	var $engine;
@@ -504,7 +702,7 @@ class PokoXRenderer
  * PokoXȡ꡼Ѵݡ˥饹
  *
  * @author Akira Kaze <feng-jing-gsyc-2s@glamenv-septzen.net>
- * @package pokox
+ * @package PokoX
  * @since 1.0
  */
 /* interface or abstract */
@@ -555,7 +753,7 @@ class PokoXStory
  * PokoX⥸塼ݡ˥饹
  *
  * @author Akira Kaze <feng-jing-gsyc-2s@glamenv-septzen.net>
- * @package pokox
+ * @package PokoX
  * @since 1.0
  */
 /* interface or abstract */
@@ -584,7 +782,7 @@ class PokoXModule
  * 'function' (&$pokox, $params, $mid = null)ΥХåؿƤ֡
  *
  * @author Akira Kaze <feng-jing-gsyc-2s@glamenv-septzen.net>
- * @package pokox
+ * @package PokoX
  * @since 1.0
  */
 class PokoXUserFuncModule extends PokoXModule
@@ -614,10 +812,10 @@ class PokoXUserFuncModule extends PokoXModule
 /**
  * PokoXѥƥȸvar_dump()ʰץ<br />
  * assign줿ͤǸvar_dumpľܽϤޤƥץ졼ȥ꥽
- * Ȥޤ
+ * ȤޤRendererClassꤵʤΥߡȤƤѤޤ
  *
  * @author Akira Kaze <feng-jing-gsyc-2s@glamenv-septzen.net>
- * @package pokox
+ * @package PokoX
  * @since 1.0
  */
 class PokoXVarDumpRenderer extends PokoXRenderer
@@ -625,7 +823,7 @@ class PokoXVarDumpRenderer extends PokoXRenderer
 	var $engine;
 	var $templates;
 
-	function &PokoXVarDumpRenderer()
+	function PokoXVarDumpRenderer()
 	{
 		$this->templates = array();
 	}
@@ -654,7 +852,7 @@ class PokoXVarDumpRenderer extends PokoXRenderer
  * ץPHP(HTML)߼Υƥץ졼ȥ饹
  *
  * @author Akira Kaze <feng-jing-gsyc-2s@glamenv-septzen.net>
- * @package pokox
+ * @package PokoX
  * @since 1.0
  */
 class PokoXSimpleRenderer extends PokoXRenderer
@@ -663,7 +861,7 @@ class PokoXSimpleRenderer extends PokoXRenderer
 	 * ƥץ졼ȤݻϢ
 	 *
 	 * @access public
-	 * @type array
+	 * @var array
 	 * @since 1.0
 	 */
 	var $engine;
@@ -674,7 +872,7 @@ class PokoXSimpleRenderer extends PokoXRenderer
 	 * @access public
 	 * @since 1.0
 	 */
-	function &PokoXSimpleRenderer()
+	function PokoXSimpleRenderer()
 	{
 		$this->engine = array();
 	}
