@@ -9,17 +9,17 @@
  * distributed with this source code. If the COPYRIGHT.txt file is missing, 
  * please visit the PokoX homepage: http://www.glamenv-septzen.net/
  *
- * $Id: PlainStory.class.php,v 1.3 2005/07/10 20:59:22 fenjin Exp $
+ * $Id: PlainStory.class.php,v 1.5 2005/08/11 09:01:05 fenjin Exp $
  */
 
 /**
- * <pre>
  * PlainStory ȡ꡼饹⥸塼
  *
+ * <pre>
  * ⤻Τޤޥ롼롣åѤʤ
  * 󡦥Ȥ⤻åѤʤץWebApp
  *
- * $_POKOX["pokox.aci"] ̤ξ硢start()"*"˥åȤ롣
+ * $_POKOX[POKOX_ACI_KEY] ̤ξ硢start()"*"˥åȤ롣
  * (̤ξǤäơץ¦ȼꤷƤ硢뤤
  *  Ǥ"*"åȤƤϾ񤭤ʤ)
  * </pre>
@@ -31,10 +31,6 @@
 class PlainStory extends PokoXStory
 {
 
-	function &PlainStory()
-	{
-	}
-
 	function start(&$pokox)
 	{
 		global $_POKOX;
